package com.topsunit.scanservice.ximai.service;

import com.topsunit.scanservice.ximai.dao.InvmbDao;
import com.topsunit.scanservice.ximai.dao.InvmlDao;
import com.topsunit.scanservice.ximai.dto.*;
import com.topsunit.scanservice.ximai.dto.mapper.InvmbMapper;
import com.topsunit.scanservice.ximai.entity.Copma;
import com.topsunit.scanservice.ximai.entity.Invmb;
import com.topsunit.scanservice.ximai.entity.Invml;
import jdk.nashorn.internal.parser.DateParser;
import org.apache.commons.lang.StringUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.math.BigDecimal;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * <p>Title: InvmbService</p>
 * <p>Description: 品号基本信息档</p>
 *
 * @author xi.feng
 * @version V1.0
 * @date 2021/11/11
 */
@Service
public class InvmbService {
    private final InvmbDao invmbDao;
    private final InvmlDao invmlDao;
    private final InvmbMapper invmbMapper;

    public InvmbService(InvmbDao invmbDao, InvmlDao invmlDao, InvmbMapper invmbMapper) {
        this.invmbDao = invmbDao;
        this.invmlDao = invmlDao;
        this.invmbMapper = invmbMapper;
    }

    public Optional<InvmbDto> getForInvmbInfo(InvmbIdCriteria criteria) {
        return invmbDao.findById(criteria.getMb001())
                .map(invmbMapper::toInvmbDto)
                .map(i -> {
                    i.setInvmls(invmbMapper.toInvmlDto(
                            invmlDao.findByMl001OrderByMl002AscMl003AscMl004Asc(criteria.getMb001())
                                    .stream()
                                    .filter(j -> j.getMl005().compareTo(BigDecimal.ZERO) != 0)
                                    .collect(Collectors.toList())
                    ));
                    return i;
                });
    }

    public List<InvmbDto> getInvmbList(InvmbCriteria criteria) {
        Specification<Invmb> query = new Specification<Invmb>() {
            @Override
            public Predicate toPredicate(Root<Invmb> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                List<Predicate> predicates = new ArrayList<Predicate>();
                if (StringUtils.isNotBlank(criteria.getMb001())) {
                    Predicate predicate = criteriaBuilder.like(root.get("mb001"), criteria.getMb001() + "%");
                    predicates.add(predicate);
                }
                if (StringUtils.isNotBlank(criteria.getMb109())) {
                    Predicate predicate = criteriaBuilder.like(root.get("mb109"), criteria.getMb109() + "%");
                    predicates.add(predicate);
                }
                if (StringUtils.isNotBlank(criteria.getGtEqCreateDate())) {
                    Predicate predicate = criteriaBuilder.greaterThanOrEqualTo(root.get("createDate"), criteria.getGtEqCreateDate());
                    predicates.add(predicate);
                }
                if (StringUtils.isNotBlank(criteria.getGtEqModiDate())) {
                    Predicate predicate = criteriaBuilder.greaterThanOrEqualTo(root.get("modiDate"), criteria.getGtEqModiDate());
                    predicates.add(predicate);
                }
                return criteriaQuery.where(predicates.toArray(new Predicate[0])).getRestriction();
            }
        };
        Page<Invmb> list = invmbDao.findAll(query, criteria.toPageable());
        List<InvmbDto> rstList = invmbMapper.toInvmbDtoList(list.toList());
        List<String> mb001List = rstList.stream().map(s->s.getMb001()).collect(Collectors.toList());
        Map<String, List<Invml>> invmlMap = invmlDao.findByMl001InOrderByMl002AscMl003AscMl004Asc(mb001List).stream().collect(Collectors.groupingBy(s->s.getMl001()));
        rstList.forEach(i->{
            List<Invml> tempList = invmlMap.getOrDefault(i.getMb001(), new ArrayList<Invml>());
            i.setInvmls(invmbMapper.toInvmlDto(tempList));
        });
        return rstList;
    }

    public List<InvmlDto> getInvmlList(InvmlCriteria criteria) {
        Specification<Invml> query = new Specification<Invml>() {
            @Override
            public Predicate toPredicate(Root<Invml> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {
                List<Predicate> predicates = new ArrayList<Predicate>();
                if (StringUtils.isNotBlank(criteria.getMl001())) {
                    Predicate predicate = criteriaBuilder.equal(root.get("ml001"), criteria.getMl001() );
                    predicates.add(predicate);
                }
                if (StringUtils.isNotBlank(criteria.getMl002())) {
                    Predicate predicate = criteriaBuilder.equal(root.get("ml002"), criteria.getMl002());
                    predicates.add(predicate);
                }
                return criteriaQuery.where(predicates.toArray(new Predicate[0])).getRestriction();
            }
        };
        List<Invml> list = invmlDao.findAll(query);
        return invmbMapper.toInvmlDto(list);
    }
}
